
import re
from japana.word_count import word_count
import click

class DeckBuilder:
    def __init__(self, glosser):
        self.glosser = glosser

    def get_sentences_from_file(self, path, jlpt, threshold, furiganadelim):
        click.echo('Reading file: '+path)
        file = open(path, "r")
        string = file.read()

        if len(furiganadelim) != 0 and len(furiganadelim) != 2:
            raise Exception("Must be 0 or 2 characters for furigana delimiters")

        if furiganadelim:
            reg = (furiganadelim[0], '.+?', furiganadelim[1])
            string = re.sub(''.join(reg), '', string)
        sentences = re.split('\n|。', string)

        click.echo('Analysing text for word frequency:')
        words = word_count(string, True, False, False)
        self.glosser.populate_ignore_set(words, jlpt, threshold)

        return sentences

    def process_sentences(self, sentences, cardGenerator):
        click.echo('Processing sentence definitions:')
        with click.progressbar(sentences) as bar:
            for sentence in bar:
                if sentence.isspace():
                    continue

                sentence = sentence.strip()

                glosses = self.glosser.fetch_glosses(sentence)

                definitions = altreadings = knowndefs = ''
                altreadings, definitions, knowndefs = self.process_glosses(altreadings, definitions, glosses, knowndefs)

                definitions = definitions[10::]
                knowndefs = knowndefs[10::]

                cardGenerator.add_card(sentence, "", definitions, altreadings, knowndefs)

    def process_glosses(self, altreadings, definitions, glosses, knowndefs):
        for gloss in glosses:
            try:
                gloss = self.glosser.remove_dict_annotations(gloss)

                readings = self.glosser.get_readings(gloss)
                altreadings += self.glosser.generate_alt_readings(readings)

                gloss = self.glosser.clean_front(gloss)

                gloss = self.glosser.clean_verb_stem(gloss)
                gloss = self.glosser.clean_back(gloss)
                gloss = self.glosser.remove_furigana(gloss)
            except Exception as e:
                print("Error with term: " + gloss)
                raise e

            if self.glosser.is_known_word(gloss):
                knowndefs += '</br></br>' + gloss
            else:
                definitions += '</br></br>' + gloss
                self.glosser.remember_word(gloss)
        return altreadings, definitions, knowndefs



import click

from deckbuilder.Gloss import Gloss
from deckbuilder.CardGenerator import CardGenerator
from deckbuilder.DeckBuilder import DeckBuilder


@click.command()
@click.argument('filepath')
@click.option('-output', default='deck', help='output filename')
@click.option('-deckid', default=-1, help='an integer than should be unique (i.e. no other deck has the same one). If '
                                          'you don\'t specify, a random one will be generated with no guarantee of '
                                          'uniqueness')
@click.option('-jlpt', default=5, help='filters out cards below the given JLPT level from the main gloss field. e.g. '
                                       '"2" will filter out words from the N3, N4, and N5 sets')
@click.option('-threshold', default=100, help='filters out cards that occur more times than this in the document from '
                                              'the main gloss field')
@click.option('-furiganadelim', default='《》', help='the two characters that indicate furigana in the text. All '
                                                   'instances of these characters and everything inside them will be '
                                                   'deleted')
def create_cards(filepath, output, deckid, jlpt, threshold, furiganadelim):
    cardGen = CardGenerator(output, deckid)
    glosser = Gloss()

    deckBuilder = DeckBuilder(glosser)
    sentences = deckBuilder.get_sentences_from_file(filepath, jlpt, threshold, furiganadelim)
    deckBuilder.process_sentences(sentences, cardGen)

    cardGen.output_deck(output+".apkg")


if __name__ == '__main__':
    create_cards()


from setuptools import setup, find_packages

# reading long description from file
with open('DESCRIPTION.txt') as file:
    long_description = file.read()

# specify requirements of your package here
REQUIREMENTS = ['genanki', 'click', 'japana']

# some more details
CLASSIFIERS = [
    'Development Status :: 4 - Beta',
    'Intended Audience :: End Users/Desktop',
    'Intended Audience :: Developers',
    'Topic :: Text Processing',
    'License :: CC0 1.0 Universal (CC0 1.0) Public Domain Dedication',
    'Programming Language :: Python :: 3',
]

# calling the setup function
setup(name='Japanese-Deck-Builder',
      version='1.0.0',
      packages=find_packages(),
      entry_points={
          'console_scripts': [
              'deckbuilder = deckbuilder.__main__:create_cards'
          ]
      },
      description='A tool for building Anki decks from Japanese text with well-formatted definitions',
      long_description=long_description,
      url='https://github.com/sduncan56/japanesedeckbuilder',
      author='Shane Duncan',
      author_email='shane.duncan56@gmail.com',
      license='CC0',
      classifiers=CLASSIFIERS,
      install_requires=REQUIREMENTS,
      keywords='anki japanese'
      )



from urllib import (
    error,
    parse,
    request,
)
import re


class Gloss:
    mirrors = ['http://nihongo.monash.edu/cgi-bin/wwwjdic?9ZIH%s', 'http://wwwjdic.biz/cgi-bin/wwwjdic?9ZIH%s']

    junk_terms = ['(P);', '(P)', 'ED;', 'ED', 'KD', 'ES', 'ST', 'SP', 'MA', 'EV', '(n,adj-no)',
                  '(v5r,vi,aux-v)', '(n)', '(adv-no)', '(adj-no,n)', '(adj-no)',
                  '(v1)', '(vi)', '(v1,vi)', '( adj-i)', '(adj-na)', '(aux)',
                  '(aux-v)', '(uk)', '(n-t)', '(adj-no,adj-na,n)']

    ignore_set = set()

    def populate_ignore_set(self, frequency_list, jlpt, freq_threshold):
        for entry in frequency_list:
            allowWord = entry['frequency'] < freq_threshold

            level = entry['jlpt']
            if level is not None:
                if int(level[1]) > jlpt:
                    allowWord = False

            if not allowWord:
                continue

            self.ignore_set.add(entry['word'])

    def remember_word(self, gloss):
        wordIndex = gloss.find(' ')
        self.ignore_set.add(gloss[0:wordIndex])

    def send_request(self, url, term, tries):
        out = None
        try:
            url = url % parse.quote(term)
            out = request.urlopen(url).read()
        except error.HTTPError:
            if tries + 1 > len(self.mirrors):
                raise ConnectionError("Cannot connect to any mirror")
            return self.send_request(self.mirrors[tries + 1], term, tries + 1)
        return out

    def fetch_glosses(self, term):
        out = self.send_request(self.mirrors[0], term, 0)

        result = out.decode("utf8")
        ls = re.findall('<li>(.*?)</li>', result)
        return ls

    def clean_verb_stem(self, gloss):
        txt = '《verb stem》'
        verbSectIndex = gloss.find(txt)
        if verbSectIndex != -1:
            numSecStart = gloss[verbSectIndex::].find('(1)')
            if numSecStart != -1:
                return gloss[0:verbSectIndex] + gloss[verbSectIndex + numSecStart::]

            for counter, c in enumerate(gloss[verbSectIndex + len(txt)::]):
                charVal = ord(c)
                if (charVal >= 65 and charVal <= 90) or (charVal >= 97 and charVal <= 122):
                    return gloss[0:verbSectIndex] + gloss[verbSectIndex + len(txt) + counter::]

        return gloss

    def clean_front(self, gloss):
        gloss = gloss.strip()

        lineBreakIndex = gloss.find('<br>')
        if lineBreakIndex != -1:
            gloss = gloss[lineBreakIndex + 4::]

        firstSpaceIndex = gloss.find(' ')
        if gloss[firstSpaceIndex::].find('from') == 1:
            gloss = gloss[firstSpaceIndex + 6::]
            if gloss[0] == ":":
                gloss = gloss[1::]
            gloss = gloss.lstrip()

        tabIndex = gloss.find('\t')
        cutIndex = -1

        seenOpenBracket = False

        for i in range(len(gloss)):
            if i + tabIndex + 1 < len(gloss):
                character = gloss[i + tabIndex + 1]
                if character.isalpha() and not character.isspace() and not seenOpenBracket:
                    tabIndex = i + tabIndex
                    break

                if character == '(':
                    seenOpenBracket = True
                elif character == ')':
                    seenOpenBracket = False

        for i in range(len(gloss)):
            if gloss[i] == ' ' or gloss[i] == ':' or gloss[i] == ';':
                cutIndex = i
                break

        assert (cutIndex > 0)

        # if there is no convenient tab, use the first english character (or '{') instead.
        # argument could be made that we should do this anyway
        if tabIndex == -1:
            for i in range(cutIndex + 1, len(gloss)):
                c = gloss[i]
                charOrd = ord(gloss[i])
                if charOrd >= 65 and charOrd <= 90 or charOrd >= 97 and charOrd <= 123:
                    tabIndex = i - 1
                    break

        if gloss[tabIndex] == '{' or gloss[tabIndex] == '(':
            gloss = gloss[0:tabIndex] + ' ' + gloss[tabIndex::]

        return gloss[0:cutIndex] + ' - ' + gloss[tabIndex + 1::]

    def remove_dict_annotations(self, gloss):
        for term in self.junk_terms:
            index = 0

            while index != -1:
                index = gloss.find(term)
                if index != -1:
                    gloss = gloss[0:index] + gloss[index + len(term)::]

        return gloss

    def clean_back(self, gloss):
        semicolonIndex = -1
        for i in range(len(gloss)):
            curIndex = len(gloss) - i - 1

            if gloss[curIndex] == ';':
                semicolonIndex = curIndex
                break

        cleaned = gloss[0:semicolonIndex]
        return cleaned

    def get_text_before_reading(self, gloss, start):
        i = start
        seenOneSpace = False
        while i > 0:
            if gloss[i].isspace():
                if i - 2 >= 0 and gloss[i - 1] == ';' and not gloss[i - 2].isspace() and ord(gloss[i - 2]) < 128:
                    return gloss[i:start].lstrip()

                elif not seenOneSpace:
                    seenOneSpace = True
                elif i - 2 >= 0 and gloss[i - 1] == ';' and ord(gloss[i - 2]) > 128:
                    seenOneSpace = False
                else:
                    return gloss[i:start].lstrip()
            elif gloss[i] == ':':
                return gloss[i + 1:start].lstrip()
            i -= 1
        return gloss[i:start]

    def get_readings(self, gloss):
        readings = []
        for i, c in enumerate(gloss):
            if c == '【':
                prevText = self.get_text_before_reading(gloss, i)
                endIndex = gloss.find('】', i)
                readings.append(prevText + gloss[i:endIndex + 1])

        return readings

    def generate_alt_readings(self, readings):
        if len(readings) == 0:
            return ''

        altreadings = ''

        for reading in readings:
            altreadings = altreadings + ' | ' + reading
        return altreadings[3::] + '</br>'

    def remove_furigana(self, gloss):
        result = ''
        lastGoodIndex = 0
        for i, c in enumerate(gloss):
            if c == '【':
                endIndex = gloss.find('】', i)
                result += gloss[lastGoodIndex:i]
                lastGoodIndex = endIndex + 1
        result += gloss[lastGoodIndex::]

        return result

    def is_known_word(self, gloss):
        wordIndex = gloss.find(' ')
        return True if gloss[0:wordIndex] in self.ignore_set else False



import unittest
from deckbuilder.Gloss import Gloss
class TestGloss(unittest.TestCase):
    def test_gloss(self):
        gloss = Gloss()
        result = gloss.fetch_glosses('よく晴れた夜空')
        self.assertEqual(len(result), 3)

    def test_clean_gloss_front(self):
        gloss = Gloss()

        text1 = ' 夜空 【よぞら】 	(n) night sky; (P); ED; Name(s):  【やくう】 (f) Yakuu 【よぞら】 (f) Yozora  SrcHNA '
        clean1 = gloss.clean_front(text1)
        self.assertEqual('夜空 - night sky; (P); ED; Name(s):  【やくう】 (f) Yakuu 【よぞら】 (f) Yozora  SrcHNA', clean1)

        text2 = ' 晴れ : 晴れ(P); 晴; 霽れ 【はれ】 	(n,adj-no) (1) (See 快晴・かいせい) clear weather; fine weather;'
        clean2 = gloss.clean_front(text2)
        self.assertEqual('晴れ - clear weather; fine weather;', clean2)

    def test_clean_front_particle(self):
        gloss = Gloss()
        text = "帝国の from 帝国 【ていこく】 (n) (1) empire; (adj-no) (2) imperial; (P); ED"
        text = gloss.remove_dict_annotations(text)
        text = gloss.clean_front(text)
        text = gloss.clean_verb_stem(text)
        text = gloss.clean_back(text)
        text = gloss.remove_furigana(text)

        self.assertEqual("帝国 - empire;  (2) imperial", text)

    def test_clean_front_particle2(self):
        gloss = Gloss()
        text = "異形の from : 異形; 異型 【いけい】 (n,adj-no) atypical appearance; atypicality; heteromorphy; 【いぎょう】 ; (adj-no,adj-na,n) fantastic; grotesque; strange-looking; suspicious-looking; ED "
        text = gloss.remove_dict_annotations(text)
        text = gloss.clean_front(text)
        text = gloss.clean_verb_stem(text)
        text = gloss.clean_back(text)
        text = gloss.remove_furigana(text)

        self.assertEqual("異形 - atypical appearance; atypicality; heteromorphy;  ;  fantastic; grotesque; strange-looking; suspicious-looking", text)

    def test_clean_front_retain_brace(self):
        gloss = Gloss()
        text = "改ページ : 改ページ; 改頁 【かいページ】 	 {comp} repagination; new page; form feed; page break;"
        text = gloss.remove_dict_annotations(text)
        text = gloss.clean_front(text)
        text = gloss.clean_verb_stem(text)
        text = gloss.clean_back(text)
        text = gloss.remove_furigana(text)

        self.assertEqual("改ページ - {comp} repagination; new page; form feed; page break", text)

    def test_clean_verb_stem_withnumbers(self):
        gloss = Gloss()
        result = gloss.clean_verb_stem('《verb stem》 晴れる : 晴れる(P); 霽れる 【はれる】 ; (v1,vi) (1) to clear up')
        self.assertEqual('(1) to clear up', result)

    def test_clean_verb_stem_withoutnumbers(self):
        gloss = Gloss()
        inputText = '《verb stem》 晴れる : 晴れる(P); 霽れる 【はれる】 ; (v1,vi) to clear up'
        #need to clear junk like '(P)' so we find the actual text start
        inputText = gloss.remove_dict_annotations(inputText)
        result = gloss.clean_verb_stem(inputText)
        self.assertEqual('to clear up', result)

    def test_clean_gloss_back(self):
        gloss = Gloss()

        text1 = ' thank you for ...; KD '
        clean1 = gloss.clean_back(text1)
        self.assertEqual(' thank you for ...', clean1)

        text2 = '(4) to be dispelled; to be banished; (P); ED '
        text2 = gloss.remove_dict_annotations(text2)
        clean2 = gloss.clean_back(text2)
        self.assertEqual('(4) to be dispelled; to be banished', clean2)

        text3 = ' 夜空 【よぞら】 	(n) night sky; (P); ED; Name(s):  【やくう】 (f) Yakuu 【よぞら】 (f) Yozora  SrcHNA '
        text3 = gloss.remove_dict_annotations(text3)
        clean3 = gloss.clean_back(text3)
        self.assertEqual(' 夜空 【よぞら】 	 night sky', clean3)

    def test_cleanall_1(self):
        gloss = Gloss()

        text = ' よく  	(adv) (1) nicely; properly; well; skillfully; skilfully; (2) (uk) frequently; often; (3) (uk) I\'m glad that you ...; thank you for ...; KD '
        clean = gloss.remove_dict_annotations(text)
        clean = gloss.clean_front(clean)
        clean = gloss.clean_verb_stem(clean)
        clean = gloss.clean_back(clean)

        self.assertEqual('よく - nicely; properly; well; skillfully; skilfully; (2)  frequently; often; (3)  I\'m glad that you ...; thank you for ...', clean)

    def test_cleanall_2(self):
        gloss = Gloss()

        text = ' 晴れ : 晴れ(P); 晴; 霽れ 【はれ】 	(n,adj-no) (1) (See 快晴・かいせい) clear weather; fine weather; (adj-no,n) (2) (ant: 褻) formal; ceremonial; public; (3) cleared of suspicion; (P);  《verb stem》 晴れる : 晴れる(P); 霽れる 【はれる】 ; (v1,vi) (1) to clear up; to clear away; to be sunny; to stop raining; (2) to refresh (e.g. spirits); (3) (See 疑いが晴れる) to be cleared (e.g. of a suspicion); (4) to be dispelled; to be banished; (P); ED '
        clean = gloss.remove_dict_annotations(text)
        clean = gloss.clean_front(clean)
        clean = gloss.clean_verb_stem(clean)
        clean = gloss.clean_back(clean)

        self.assertEqual('晴れ - clear weather; fine weather;  (2) (ant: 褻) formal; ceremonial; public; (3) cleared of suspicion;   (1) to clear up; to clear away; to be sunny; to stop raining; (2) to refresh (e.g. spirits); (3) (See 疑いが晴れる) to be cleared (e.g. of a suspicion); (4) to be dispelled; to be banished', clean)

    def test_cleanall_3(self):
        gloss = Gloss()

        text = ' 夜空 【よぞら】 	(n) night sky; (P); ED; Name(s):  【やくう】 (f) Yakuu 【よぞら】 (f) Yozora  SrcHNA '
        clean = gloss.remove_dict_annotations(text)
        clean = gloss.clean_front(clean)
        clean = gloss.clean_verb_stem(clean)
        clean = gloss.clean_back(clean)

        self.assertEqual('夜空 - night sky', clean)

    def test_get_readings(self):
        gloss = Gloss()

        text = "頭 【あたま(P); かしら(P)】 (n) (1) head; (2) hair (on one's head); (3) (あたま only) mind; brains; intellect; (4) leader; chief; boss; captain; (5) top; tip; (6) beginning; start; (7) (あたま only) head; person; (8) (かしら only) top structural component of a kanji; (9) (あたま only) (col) {mahj} (See 雀頭・ジャントー) pair; (P); 【とう】 ; (ctr) counter for large animals (e.g. head of cattle); counter for insects in a collection; counter for helmets, masks, etc.; (P);  : 頭; 首 【こうべ; かぶり(頭); ず(頭); つむり(頭); つむ(頭); つぶり(頭)(ok); かぶ(頭)(ok)】 ; (n) head; : ど頭; 頭 【どたま】 ; (n) (uk) (derog) head; dome; bean; nob; noggin; 【かぶし】 ; (n) (arch) (uk) shape of one's head; 【がしら】 ; (suf) (1) (after a noun) top of ...; head of ...; (2) (after the -masu stem of a verb) the moment that ...;"

        readings = gloss.get_readings(text)
        self.assertEqual(6, len(readings))
        self.assertEqual('頭 【あたま(P); かしら(P)】', readings[0])
        self.assertEqual('【とう】', readings[1])
        self.assertEqual('頭; 首 【こうべ; かぶり(頭); ず(頭); つむり(頭); つむ(頭); つぶり(頭)(ok); かぶ(頭)(ok)】', readings[2])
        self.assertEqual('ど頭; 頭 【どたま】', readings[3])
        self.assertEqual('【かぶし】', readings[4])
        self.assertEqual('【がしら】', readings[5])

    def test_generate_alt_readings(self):
        gloss = Gloss()
        text = "頭 【あたま(P); かしら(P)】 (n) (1) head; (2) hair (on one's head); (3) (あたま only) mind; brains; intellect; (4) leader; chief; boss; captain; (5) top; tip; (6) beginning; start; (7) (あたま only) head; person; (8) (かしら only) top structural component of a kanji; (9) (あたま only) (col) {mahj} (See 雀頭・ジャントー) pair; (P); 【とう】 ; (ctr) counter for large animals (e.g. head of cattle); counter for insects in a collection; counter for helmets, masks, etc.; (P);  : 頭; 首 【こうべ; かぶり(頭); ず(頭); つむり(頭); つむ(頭); つぶり(頭)(ok); かぶ(頭)(ok)】 ; (n) head; : ど頭; 頭 【どたま】 ; (n) (uk) (derog) head; dome; bean; nob; noggin; 【かぶし】 ; (n) (arch) (uk) shape of one's head; 【がしら】 ; (suf) (1) (after a noun) top of ...; head of ...; (2) (after the -masu stem of a verb) the moment that ...;"

        text = gloss.remove_dict_annotations(text)
        readings = gloss.get_readings(text)
        altreadings = gloss.generate_alt_readings(readings)

        self.assertEqual('頭 【あたま かしら】 | 【とう】 | 頭; 首 【こうべ; かぶり(頭); ず(頭); つむり(頭); つむ(頭); つぶり(頭)(ok); かぶ(頭)(ok)】 | ど頭; 頭 【どたま】 | 【かぶし】 | 【がしら】</br>', altreadings)

    def test_generate_alt_readings_withparticle(self):
        gloss = Gloss()

        text = '帝国の from 帝国 【ていこく】 (n) (1) empire; (adj-no) (2) imperial; (P); ED'
        text = gloss.remove_dict_annotations(text)
        readings = gloss.get_readings(text)
        altreadings = gloss.generate_alt_readings(readings)
        self.assertEqual('帝国 【ていこく】</br>', altreadings)

    def test_generate_alt_readings_noreading(self):
        gloss = Gloss()

        text = "ここから (exp) from here; KD"
        readings = gloss.get_readings(text)
        altreadings = gloss.generate_alt_readings(readings)
        self.assertEqual(0, len(altreadings))

    @unittest.skip("this would look nicer but I don't actually care")
    def test_get_readings_cleaned(self):
        gloss = Gloss()

        text = gloss.remove_dict_annotations("頭 【あたま(P); かしら(P)】 (n) (1) head;")

        readings = gloss.get_readings(text)
        self.assertEqual('頭 【あたま; かしら】', readings[0])

    def test_remove_furigana(self):
        gloss = Gloss()

        text = "頭 【あたま(P); かしら(P)】 (n) (1) head; (2) hair (on one's head); (3) (あたま only) mind; brains; intellect; (4) leader; chief; boss; captain; (5) top; tip; (6) beginning; start; (7) (あたま only) head; person; (8) (かしら only) top structural component of a kanji; (9) (あたま only) (col) {mahj} (See 雀頭・ジャントー) pair; (P); 【とう】 ; (ctr) counter for large animals (e.g. head of cattle); counter for insects in a collection; counter for helmets, masks, etc.; (P);  : 頭; 首 【こうべ; かぶり(頭); ず(頭); つむり(頭); つむ(頭); つぶり(頭)(ok); かぶ(頭)(ok)】 ; (n) head; : ど頭; 頭 【どたま】 ; (n) (uk) (derog) head; dome; bean; nob; noggin; 【かぶし】 ; (n) (arch) (uk) shape of one's head; 【がしら】 ; (suf) (1) (after a noun) top of ...; head of ...; (2) (after the -masu stem of a verb) the moment that ...;"

        clean = gloss.remove_furigana(text)
        self.assertEqual("頭  (n) (1) head; (2) hair (on one's head); (3) (あたま only) mind; brains; intellect; (4) leader; chief; boss; captain; (5) top; tip; (6) beginning; start; (7) (あたま only) head; person; (8) (かしら only) top structural component of a kanji; (9) (あたま only) (col) {mahj} (See 雀頭・ジャントー) pair; (P);  ; (ctr) counter for large animals (e.g. head of cattle); counter for insects in a collection; counter for helmets, masks, etc.; (P);  : 頭; 首  ; (n) head; : ど頭; 頭  ; (n) (uk) (derog) head; dome; bean; nob; noggin;  ; (n) (arch) (uk) shape of one's head;  ; (suf) (1) (after a noun) top of ...; head of ...; (2) (after the -masu stem of a verb) the moment that ...;", clean)

    def test_clean_front_no_tabs(self):
        gloss = Gloss()
        text = '衛星 【えいせい】 (n) (1) {astron} (natural) satellite; moon; (2) (See 人工衛星) (artificial) satellite; (P); ED Name(s): 【えいせい】 (u) Eisei '

        clean = self.clean_all(gloss, text)

        self.assertEqual('衛星 - {astron} (natural) satellite; moon; (2) (See 人工衛星) (artificial) satellite', clean)

    def clean_all(self, gloss, text):
        clean = gloss.remove_dict_annotations(text)
        clean = gloss.clean_front(clean)
        clean = gloss.clean_verb_stem(clean)
        clean = gloss.clean_back(clean)
        clean = gloss.remove_furigana(clean)
        return clean

    def test_leading_text(self):
        gloss = Gloss()

        text = " Possible inflected verb or adjective: (passive)<br>描く : 描く(P); 画く 【えがく(P); かく】 	(v5k,vt) (1) (See 書く・2) to draw; to paint; to sketch; (2) (えがく only) to depict; to describe; (3) to picture in one's mind; to imagine; (4) to form a certain shape (e.g. path of an action, appearance of an object, etc.); (P); ED "

        clean = self.clean_all(gloss, text)
        self.assertEqual("描く - to draw; to paint; to sketch; (2) (えがく only) to depict; to describe; (3) to picture in one's mind; to imagine; (4) to form a certain shape (e.g. path of an action, appearance of an object, etc.)", clean)

    def test_should_ignore_gloss(self):
        gloss = Gloss()
        gloss.ignore_set.add("生活")
        text = "生活 【せいかつ】 (n,vs) living; life (one's daily existence); livelihood; (P); ED "
        shouldIgnore = gloss.is_known_word(text)
        self.assertTrue(shouldIgnore)

    def test_populate_ignore_set_jlpt(self):
        gloss = Gloss()
        frequency = [{'word': 'tst', 'jlpt': 'N2', 'frequency': 1},
                     {'word': 'tst2', 'jlpt': 'N4', 'frequency': 1}]
        gloss.populate_ignore_set(frequency, 3, 100)
        self.assertEqual('tst', gloss.ignore_set.pop())
        self.assertEqual(len(gloss.ignore_set), 0)




import genanki
from random import randint

class CardGenerator:
    def __init__(self, deckName, deckId):

        if deckId == -1:
            deckId = randint(0, 9999999999)

        self.model = genanki.Model(1878474795,
                                   'Japanese book v2',
                                   fields=[
                                       {'name': 'Expression'},
                                       {'name': 'Reading'},
                                       {'name': 'Gloss'},
                                       {'name': 'AltReadings'},
                                       {'name': 'Known Definitions'}
                                   ],
                                   templates=[
                                       {
                                           'name': 'Japanese Book',
                                           'qfmt': ' <p style="font-size:30px;text-align:center"><span class=jp>{{Expression}}</span></p>',
                                           'afmt': '<hr id="answer"><p style="font-size:30px;text-align:center"> {{furigana:Reading}}</p><p style="font-size:20px">{{furigana:Gloss}}</p>',
                                       },
                                   ])
        self.deck = genanki.Deck(
            deckId,
            deckName
        )

    def add_card(self, expression, reading, gloss, altreadings, knowndefs):
        note = genanki.Note(
            model=self.model,
            fields=[expression, reading, gloss, altreadings, knowndefs],
        )
        self.deck.add_note(note)

    def output_deck(self, path):
        self.deck.write_to_file(path)